package loja.ejb;

import javax.ejb.*;
import java.util.*;
import java.math.BigDecimal;

import loja.integration.*;
import loja.*;

/**
 * @ejb:bean
 		name="Produto"
		jndi-name="ejb/loja-simples/produto"
		type="BMP"
		view-type="both"
 * @ejb:dao
 		class="loja.integration.ProdutoDAO"
		impl-class="loja.integration.JdbcProdutoDAO"
 * @ejb:transaction type="Required"
 * @ejb:resource-ref
 		res-name="jdbc/LojaDB"
		res-type="javax.sql.DataSource"
		res-auth="Container"
 * @ejb:data-object package="loja"
 *
 * @jboss:resource-manager
 		res-man-class="javax.sql.DataSource"
 		res-man-name="jdbc/LojaDB"
		res-man-jndi-name="java:/DefaultDS"
 */
 	
public class ProdutoBean implements EntityBean {
	
	protected EntityContext ctx;
	
	private String codigo;
	private String nome;
	private BigDecimal preco;
	private int quantidade;
	
	private ProdutoDAO produtoDAO;
	
	//////////////////// life-cycle ////////////////////////////
	
	/////////// 1. HOME ///////////////
	public void  ejbPostCreate(String codigo) {}
	/**
	 * @ejb:create-method
	 */
	public ProdutoPK ejbCreate(String codigo) throws CreateException {
		return ejbCreate(codigo, "", new BigDecimal(0.0), 0);
	}
	
	public void  ejbPostCreate(String codigo, String nome, BigDecimal preco) {}
	/**
	 * @ejb:create-method
	 */
	public ProdutoPK ejbCreate(String codigo, String nome, BigDecimal preco) throws CreateException {
		return ejbCreate(codigo, nome, preco, 0);
	}
	
	public void  ejbPostCreate(String codigo, String nome, BigDecimal preco, int quantidade) {}
	/**
	 * @ejb:create-method
	 */
	public ProdutoPK ejbCreate(String codigo, String nome, BigDecimal preco, int quantidade) throws CreateException {
		System.out.println("ProdutoBean: ejbCreate("+codigo+","+ nome+","+ preco+","+ quantidade+")");
		this.codigo = codigo;
		this.nome = nome;
		this.preco = preco;
		this.quantidade = quantidade;
		try {
			produtoDAO = JdbcProdutoDAO.getInstance();
			ProdutoVO produto = new ProdutoVO(codigo, nome, preco, quantidade);
			return new ProdutoPK(produtoDAO.create(produto));
		} catch (LojaException e) {
			throw new CreateException("Incapaz de obter instancia DAO: " + e);
		}
	}
	
	public ProdutoPK ejbFindByPrimaryKey(ProdutoPK pk) throws FinderException {
		System.out.println("ProdutoBean: ejbFindByPrimaryKey()");
		try {
			produtoDAO = JdbcProdutoDAO.getInstance();
			if (produtoDAO.exists(pk.codigo)) {
				return pk;
			} else {
				throw new ObjectNotFoundException("Objeto " + pk.codigo + " nao existe!");
			}
		} catch (LojaException e) {
			throw new EJBException("Falha na comunicacao com o banco: " + e);
		}
	}
	
	/**
	 * @ejb:finder signature="java.util.Collection findAll()"
	 */
	public Collection ejbFindAll() throws FinderException {
		System.out.println("ProdutoBean: ejbFindAll()");
		String[] codigos = null;
		try {
			produtoDAO = JdbcProdutoDAO.getInstance();
			codigos = produtoDAO.getAll();
		} catch (LojaException e) {
			throw new EJBException("Falha na comunicacao com o banco: " + e);
		}
		List list = new ArrayList(codigos.length);
		for (int i = 0; i < codigos.length; i++) {
			list.add(new ProdutoPK(codigos[i]));
		}
		return list;
	}
	
	/////////// 2. CONTAINER CALL-BACK ///////////////////
	public void ejbLoad() {
		System.out.println("ProdutoBean: ejbLoad()");
		ProdutoPK pk = (ProdutoPK)ctx.getPrimaryKey(); 
		ProdutoVO produto = null;
		try {
			produtoDAO = JdbcProdutoDAO.getInstance();
			produto = produtoDAO.load(pk.codigo);
		} catch (LojaException e) {
			throw new EJBException("Falha na comunicacao com o banco: " + e);
		}
		// Nao use setXXX() para nao causar a execucao de store
		codigo = produto.getCodigo();
		nome   = produto.getNome();
		preco  = produto.getPreco();
		quantidade = produto.getQuantidade();
	}
	
	public void ejbStore() {
		System.out.println("ProdutoBean: ejbStore()");
		ProdutoVO produto = new ProdutoVO(codigo, nome, preco, quantidade);
		try {
			produtoDAO = JdbcProdutoDAO.getInstance();
			produtoDAO.store(produto);
		} catch (LojaException e) {
			throw new EJBException("Falha na comunicacao com o banco: " + e);
		}
	}
	
	public void ejbRemove() {
		System.out.println("ProdutoBean: ejbRemove()");
		ProdutoPK pk = (ProdutoPK)ctx.getPrimaryKey();
		try {
			produtoDAO = JdbcProdutoDAO.getInstance();
			produtoDAO.remove(pk.codigo);
		} catch (LojaException e) {
			throw new EJBException("Falha na comunicacao com o banco: " + e);
		}
	}
	
	public void setEntityContext(EntityContext ctx) {
		System.out.println("ProdutoBean: setEntityContext()");
		this.ctx = ctx;
	}
	public void unsetEntityContext() {
		ctx = null;
	}
	
	public void ejbActivate() {}
	public void ejbPassivate() {}
	
	
	//////////// BUSINESS AND ACESSOR ///////////////
	
	/**
	 * @ejb:persistent-field
	 * @ejb:interface-method
	 * @ejb:pk-field
	 */
	public String getCodigo() {
		System.out.println("ProdutoBean: getCodigo()");
		return codigo;
	}
	/**
	 * @ejb:persistent-field
	 * @ejb:interface-method
	 * @ejb:pk-field
	 */
	public void setCodigo(String codigo) {
		System.out.println("ProdutoBean: setCodigo("+codigo+")");
		this.codigo = codigo;
	}
	
	/**
	 * @ejb:persistent-field
	 * @ejb:interface-method
	 */
	public void setNome(String nome) {
		System.out.println("ProdutoBean: setNome("+nome+")");
		this.nome = nome;
	}
	/**
	 * @ejb:persistent-field
	 * @ejb:interface-method
	 */
	public String getNome() {
		System.out.println("ProdutoBean: getNome()");
		return nome;
	}
	
	/**
	 * @ejb:persistent-field
	 * @ejb:interface-method
	 */
	public void setPreco(BigDecimal preco) {
		System.out.println("ProdutoBean: setPreco("+preco+")");
		this.preco = preco;
	}
	/**
	 * @ejb:persistent-field
	 * @ejb:interface-method
	 */
	public BigDecimal getPreco() {
		System.out.println("ProdutoBean: getPreco()");
		return preco;
	}
	
	/**
	 * @ejb:persistent-field
	 * @ejb:interface-method
	 */
	public int getQuantidade() {
		System.out.println("ProdutoBean: getQuantidade()");
		return quantidade;
	}
	/**
	 * @ejb:persistent-field
	 * @ejb:interface-method
	 */
	public void setQuantidade(int quantidade) {
		System.out.println("ProdutoBean: setQuantidade("+quantidade+")");
		this.quantidade = quantidade;
	}
	
	/**
	 * @ejb:persistent-field
	 * @ejb:interface-method
	 */
	public void addItens(int quantidade) {
		System.out.println("ProdutoBean: addItens("+quantidade+")");
		this.quantidade += quantidade;
	}

}
